# Module:   __init__
# Date:     3rd October 2008
# Author:   James Mills, prologic at shortcircuit dot net dot au

"""sahriswiki - Lightweight Wiki Engine

:copyright: CopyRight (C) 2010-2011 by James Mills
:license: MIT (See: LICENSE)
"""

__author__ = "James Mills"
__date__ = "31st January 2011"
__version__ = "0.8"
