<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en"
 xmlns:xi="http://www.w3.org/2001/XInclude"
 xmlns:py="http://genshi.edgewall.org/">
 <xi:include href="layout.html" />
 <head>
  <title>Editing</title>
 </head>
 <body>
  <div id="editor">
   <form method="post" action="">
    <input py:if="'node' in page" type="hidden" name="parent"
     value="${page['node']}" />
		 	<p class="text">
    <textarea id="text" name="text" rows="25"
     cols="80">${page["text"]}</textarea>
		 	</p>
		 	<h4><label for="comment">Comment:</label></h4>
		 	<p class="input">
    <input id="comment" name="comment"
     value="${value_of('comment', '')}" />
		 	</p>
		 	<p class="button">
     <button name="action" type="submit" value="preview">Preview</button>
     <button name="action" type="submit" value="save">Save</button>
     <button name="action" type="submit" value="cancel">Cancel</button>
		 	</p>
   </form>
  </div>
  <py:if test="defined('preview')">
   <py:if test="preview">
    <div id="preview"><h1>Preview</h1></div>
    ${html}
   </py:if>
  </py:if>
 </body>
</html>
