<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en"
 xmlns:xi="http://www.w3.org/2001/XInclude"
 xmlns:py="http://genshi.edgewall.org/">
 <xi:include href="layout.html" />
 <head>
  <title>${code} ${name}</title>
  <style type="text/css">
   #traceback {
    color: red;
   }
  </style>
 </head>
 <body>
  <h1>${name}</h1>
  ${description}
  <pre id="traceback" py:if="traceback">${traceback}</pre>
 </body>
</html>
