<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en"
 xmlns:xi="http://www.w3.org/2001/XInclude"
 xmlns:py="http://genshi.edgewall.org/">
 <xi:include href="layout.html" />
 <head></head>
 <body>
  <h1>${title}</h1>
  <p>${len(results)} pages(s) found matching all words.</p>
   <py:choose test="len(results) > 0">
    <py:when test="True">
     <ol>
      <li py:for="score, name, snippet in results">
       <b><a href="${url('/%s' % name)}">${name}</a></b>
       <i>${score}</i>
       <p>${snippet}</p>
      </li>
     </ol>
    </py:when>
    <py:otherwise>
     <p py:if="'PAGE_EDIT in permissions'">
      Maybe you wanted to:
      <ul>
       <li><a href="${url('/+edit', query)}">Create the Page</a></li>
       <li><a href="${url('/+upload', query)}">Upload a File</a></li>
      </ul>
     </p>
    </py:otherwise>
   </py:choose>
 </body>
</html>
